==> async-json-0.0.1.tgz: package/lib/async-json.js <==
(function (exports, undefined) {
    "use strict";
    
    if (!JSON || !JSON.stringify) {
        // older browsers
        throw new Error("The json2.js file must be included before async-json.js");
    }
    var jsonStringify = JSON.stringify;
    var isArray = Array.isArray || function (o) {
        // older browsers
        return Object.prototype.toString.call(o) === '[object Array]';
    };
    var getKeys = Object.keys || (function () {
        // older browsers
        
        var has = Object.prototype.hasOwnProperty || function () {
            // Object.prototype.hasOwnProperty should really always exist.
            return true;
        };
        
        return function (obj) {
            var result = [];
            for (var key in obj) {
                if (has.call(obj, key)) {
                    result.push(key);
                }
            }
            return result;
        };
    }());

    var stringify;
    
    /**
     * Stringify an array
     *
     * @param {Array} array The array to stringify.
     * @param {Function} callback The callback to invoke when completed the stringification.
     * @api private
     *
     * @example internalStringifyArray([1, 2, 3], function(err, value) { value === "[1,2,3]"; });
     */
    var internalStringifyArray = function (array, callback) {
        var len = array.length;
        if (len === 0) {
            callback(null, "[]");
            return;
        }
        
        // buffer is our ultimate return value
        var buffer = "[";
        
        var handle = function (n) {
            if (n === len) {
                // we're done
                buffer += "]";
                callback(null, buffer);
                return;
            }
            
            // asynchronously stringify the nth element.
            stringify(array[n], function (err, value) {
                if (err) {
                    callback(err);
                    return;
                }
 
                if (n > 0) {
                    buffer += ",";
                }
                
                if (value === undefined) {
                    // JSON.stringify turns bad values in arrays into null, so we need to as well
                    buffer += "null";
                } else {
                    buffer += value;
                }
                
                // go to the next element
                handle(n + 1);
            });
        };
        // let's pump, starting at index 0
        handle(0);
    };
    
    /**
     * Stringify an object
     *
     * @param {Object} object The object to stringify.
     * @param {Function} callback The callback to invoke when completed the stringification.
     * @api private
     *
     * @example internalStringifyObject({alpha: 1, bravo: 2}, function(err, value) { value === '{"alpha":1,"bravo":2}'; });
     */
    var internalStringifyObject = function (object, callback) {
        // getKeys _should_ be a reference to Object.keys
        // JSON.stringify gets the keys in the same order as this, but that is arbitrary.
        var keys = getKeys(object);
        var len = keys.length;
        if (len === 0) {
            callback(null, "{}");
            return;
        }
        
        // whether or not we've placed the first element in yet.
        // can't rely on i === 0, since we might skip it if the value === undefined.
        var first = true;
        
        // buffer is our ultimate return value
        var buffer = "{";
        
        var handle = function (n) {
            if (n === len) {
                buffer += "}";
                callback(null, buffer);
                return;
            }
            
            var key = keys[n];
            // asynchronously stringify the nth element in our list of keys
            stringify(object[key], function (err, value) {
                if (err) {
                    callback(err);
                    return;
                }
                
                // if we get an undefined, rather than placing in null like the array does, we just skip it.
                if (value !== undefined) {
                    if (first) {
                        first = false;
                    } else {
                        buffer += ",";
                    }
                    
                    buffer += jsonStringify(key);
                    buffer += ":";
                    buffer += value;
                }
                
                // go to the next key
                handle(n + 1);
            });
        };
        // let's pump, starting at index 0
        handle(0);
    };
    
    /**
     * Asynchronously convert a JavaScript object to JSON.
     * If any functions are supplied in the data, it will be invoked.
     * If the function has 0 parameters, it will be invoked and treated as synchronous, its return value being its replacement.
     * Otherwise, the first parameter is assumed to be a callback which should be invoked as callback(error, result)
     * 
     * @param {Any} data Any JavaScript object.
     * @param {Function} callback A callback that takes an error and the result as parameters.
     * @api public
     *
     * @example stringify({some: "data"}, function(err, value) { if (err) { throw err; } value === '{"some":"data"}' })
     */
    exports.stringify = stringify = function (data, callback) {
        if (data === undefined) {
            return callback(null, undefined);
        }
        try {
            switch (typeof data) {
            case "string":
            case "number":
                return callback(null, jsonStringify(data));
            case "boolean":
                return callback(null, data ? "true" : "false");
            case "object":
                if (data === null) {
                    // why is typeof null === "object"?
                    return callback(null, "null");
                } else if (data.constructor === String || data.constructor === Number || data.constructor === Boolean) {
                    // horrible, someone used the new String(), new Number(), or new Boolean() syntax.
                    return stringify(data.valueOf(), callback);
                } else if (isArray(data)) {
                    return internalStringifyArray(data, callback);
                } else {
                    return internalStringifyObject(data, callback);
                }
                break;
            case "function":
                if (data.length === 0) {
                    // assume a sync function that returns a value
                    return stringify(data(), callback);
                } else {
                    // assume an async function that takes a callback
                    return data(function (err, value) {
                        if (err) {
                            callback(err);
                        } else {
                            stringify(value, callback);
                        }
                    });
                }
                break;
            default:
                throw new Error("Unknown object type: " + (typeof data));
            }
        } catch (err) {
            return callback(err);
        }
    };
}(exports || (this.asyncJSON = {})));

==> async-json-0.0.2.tgz: package/lib/async-json.js <==
(function (exports, undefined) {
    "use strict";
    
    if (!JSON || !JSON.stringify) {
        // older browsers
        throw new Error("The json2.js file must be included before async-json.js");
    }
    var jsonStringify = JSON.stringify;
    var isArray = Array.isArray || function (o) {
        // older browsers
        return Object.prototype.toString.call(o) === '[object Array]';
    };
    var getKeys = Object.keys || (function () {
        // older browsers
        
        var has = Object.prototype.hasOwnProperty || function () {
            // Object.prototype.hasOwnProperty should really always exist.
            return true;
        };
        
        return function (obj) {
            var result = [];
            for (var key in obj) {
                if (has.call(obj, key)) {
                    result.push(key);
                }
            }
            return result;
        };
    }());

    var stringify;
    
    /**
     * Stringify an array
     *
     * @param {Array} array The array to stringify.
     * @param {Function} callback The callback to invoke when completed the stringification.
     * @api private
     *
     * @example internalStringifyArray([1, 2, 3], function(err, value) { value === "[1,2,3]"; });
     */
    var internalStringifyArray = function (array, callback) {
        var len = array.length;
        if (len === 0) {
            callback(null, "[]");
            return;
        }
        
        // buffer is our ultimate return value
        var buffer = "[";
        
        var handle = function (n) {
            if (n === len) {
                // we're done
                buffer += "]";
                callback(null, buffer);
                return;
            }
            
            // asynchronously stringify the nth element.
            stringify(array[n], function (err, value) {
                if (err) {
                    callback(err);
                    return;
                }
 
                if (n > 0) {
                    buffer += ",";
                }
                
                if (value === undefined) {
                    // JSON.stringify turns bad values in arrays into null, so we need to as well
                    buffer += "null";
                } else {
                    buffer += value;
                }
                
                // go to the next element
                handle(n + 1);
            }, String(n));
        };
        // let's pump, starting at index 0
        handle(0);
    };
    
    /**
     * Stringify an object
     *
     * @param {Object} object The object to stringify.
     * @param {Function} callback The callback to invoke when completed the stringification.
     * @api private
     *
     * @example internalStringifyObject({alpha: 1, bravo: 2}, function(err, value) { value === '{"alpha":1,"bravo":2}'; });
     */
    var internalStringifyObject = function (object, callback) {
        // getKeys _should_ be a reference to Object.keys
        // JSON.stringify gets the keys in the same order as this, but that is arbitrary.
        var keys = getKeys(object);
        var len = keys.length;
        if (len === 0) {
            callback(null, "{}");
            return;
        }
        
        // whether or not we've placed the first element in yet.
        // can't rely on i === 0, since we might skip it if the value === undefined.
        var first = true;
        
        // buffer is our ultimate return value
        var buffer = "{";
        
        var handle = function (n) {
            if (n === len) {
                buffer += "}";
                callback(null, buffer);
                return;
            }
            
            var key = keys[n];
            // asynchronously stringify the nth element in our list of keys
            stringify(object[key], function (err, value) {
                if (err) {
                    callback(err);
                    return;
                }
                
                // if we get an undefined, rather than placing in null like the array does, we just skip it.
                if (value !== undefined) {
                    if (first) {
                        first = false;
                    } else {
                        buffer += ",";
                    }
                    
                    buffer += jsonStringify(key);
                    buffer += ":";
                    buffer += value;
                }
                
                // go to the next key
                handle(n + 1);
            }, key);
        };
        // let's pump, starting at index 0
        handle(0);
    };
    
    /**
     * Asynchronously convert a JavaScript object to JSON.
     * If any functions are supplied in the data, it will be invoked.
     * If the function has 0 parameters, it will be invoked and treated as synchronous, its return value being its replacement.
     * Otherwise, the first parameter is assumed to be a callback which should be invoked as callback(error, result)
     * 
     * @param {Any} data Any JavaScript object.
     * @param {Function} callback A callback that takes an error and the result as parameters.
     * @api public
     *
     * @example stringify({some: "data"}, function(err, value) { if (err) { throw err; } value === '{"some":"data"}' })
     */
    exports.stringify = stringify = function (data, callback, key) {
        if (data === undefined) {
            return callback(null, undefined);
        }
        try {
            switch (typeof data) {
            case "string":
            case "number":
                return callback(null, jsonStringify(data));
            case "boolean":
                return callback(null, data ? "true" : "false");
            case "object":
                if (data === null) {
                    // why is typeof null === "object"?
                    return callback(null, "null");
                } else if (typeof data.toJSON === "function") {
                    // used by Date and possibly some others.
                    return stringify(data.toJSON(key), callback, key);
                } else if (data.constructor === String || data.constructor === Number || data.constructor === Boolean) {
                    // horrible, someone used the new String(), new Number(), or new Boolean() syntax.
                    return stringify(data.valueOf(), callback, key);
                } else if (isArray(data)) {
                    return internalStringifyArray(data, callback);
                } else {
                    return internalStringifyObject(data, callback);
                }
                break;
            case "function":
                if (data.length === 0) {
                    // assume a sync function that returns a value
                    return stringify(data(), callback, key);
                } else {
                    // assume an async function that takes a callback
                    return data(function (err, value) {
                        if (err) {
                            callback(err);
                        } else {
                            stringify(value, callback, key);
                        }
                    });
                }
                break;
            default:
                throw new Error("Unknown object type: " + (typeof data));
            }
        } catch (err) {
            return callback(err);
        }
    };
}(exports || (this.asyncJSON = {})));
